Option Explicit
Class PauserCls

' Pauser implements a simple Pause feature that uses the Pause key on the 
' keyboard to pause and resume a running script.  To use it, just add this 
' self-initializing module to your SkunkWorks project and replace some of 
' your skapi.WaitEvent(0) calls with calls to Pauser.Check().  When the 
' user presses the Pause key, a hotkey event will fire.  Next time you 
' call Pauser.Check, it will display a "Pausing script" message and loop 
' internally until the user presses Pause a second time, at which point 
' Pauser.Check will return.  Obviously you'll want to put your calls to 
' Pauser.Check in places where pausing indefinitely is OK, such as when 
' your toon is motionless and in a safe spot.
' 
' Pauser can also stop a running script on certain configurable conditions 
' listed below.  If one of these conditions arises (and the corresponding 
' configuration flag is set), Pauser.Check throws an exception up the 
' call stack.  You can catch this exception using On Error Resume Next 
' (in VBScript) or try {} catch(e) {} (in JScript).  You can retrieve 
' the text of the error message from Err.Description (in VBScript) or 
' e.description (in JScript).  If you don't catch the exception, the 
' SkunkWorks console will report it as a script error and stop your 
' script.
' 
' The supported error conditions are:
' 
' Ctrl+Break.  Set Pauser.fStopOnCtrlBreak to True to stop the script if 
' Ctrl+Break is hit, or False (the default) to ignore Ctrl+Break.  (Note 
' that Ctrl+Break detection requires SkunkWorks 2.1.)
' 
' Combat mode.  Set Pauser.fStopOnCombatMode to True to stop the script 
' if you enter combat mode, or False (the default) to ignore combat mode.
' 
' Logout.  Set Pauser.fStopOnLogout to True to stop the script if your 
' toon logs out, or False (the default) to ignore logout.
' 
' Death.  Set Pauser.fStopOnDeath to True to stop the script if your 
' toon dies, or False (the default) to ignore death.
' 
' You can also define your own error conditions by calling 
' Pauser.SetError(sz) from any event handler, where sz is a text string.  
' The next time Pauser.Check is called, it will throw an exception with 
' the given string as the error message.
' 
' If necessary, you can clear any pending error condition by calling 
' Pauser.Clear before the next call to Pauser.Check.


'-------------------
' Class boilerplate
'-------------------

Private Sub Class_Initialize()
	
	fStopOnCtrlBreak = False
	fStopOnCombatMode = False
	fStopOnLogout = False
	fStopOnDeath = False
	
	Call Clear()
	
	Call skapi.Hotkey("{Pause}", True)
	Call skapi.Hotkey("{Break}", True)
	
	Call skapi.AddHandler(evidOnStart3D, Me)
	Call skapi.AddHandler(evidOnHotkey, Me)
	Call skapi.AddHandler(evidOnCombatMode, Me)
	Call skapi.AddHandler(evidOnEnd3D, Me)
	Call skapi.AddHandler(evidOnDeathSelf, Me)
	
End Sub

Private Sub Class_Terminate()
	
	Call skapi.Hotkey("{Pause}", False)
	Call skapi.Hotkey("{Break}", False)
	
End Sub


'------------
' Properties
'------------

Public fStopOnCtrlBreak		' Raise an error if you hit Ctrl+Break.
Public fStopOnCombatMode	' Raise an error if you enter combat mode.
Public fStopOnLogout		' Raise an error if you log out.
Public fStopOnDeath			' Raise an error if you die.

' Return True if the script command should continue running, 
' False if a pause or error state is pending.
Public Property Get fRunning()
	fRunning = (pstate = pstateRunning)
End Property


'---------
' Methods
'---------

' Process events in the queue and check to see if a pause or error state is pending.
' If the user has pressed Pause, wait until Pause is pressed again before returning.
' If an error is pending, raise it now.
' Call this function periodically from your script commands instead of skapi.WaitEvent(0).
Public Function Check()
	
	Call skapi.WaitEvent()
	
	If pstate = pstatePausing Then
		Call skapi.OutputLine("Pausing script.  Press Pause again to resume.")
		pstate = pstatePaused
		
		Do While pstate = pstatePaused
			Call skapi.WaitEvent(500)
		Loop
		
		If pstate = pstateRunning Then
			Call skapi.OutputLine("Resuming script.")
		End If
	End If
	
	If pstate = pstateError Then
	'	Clear the error state before raising the error.
		Dim szT : szT = szError
		pstate = pstateRunning
		szError = ""
		Call Err.Raise(-1, "Pauser", szT)
	End If
	
	Call skapi.WaitEvent()
	
End Function

' Set a pending error condition to be raised next time Pauser.Check is called.
' sz is the string that gets passed as the error description.
' Call this from an event handler in order to break out of a running command.
Public Function SetError(sz)
	
	pstate = pstateError
	szError = sz
	
End Function

' Clear any pending pause or error states.
' Call this at the start of a script command.
Public Function Clear()
	
	pstate = pstateRunning
	szError = ""
	
End Function


'----------------
' Event handlers
'----------------

Public Function OnStart3D()
	
'	Set up our hotkeys on login.
	Call skapi.Hotkey("{Pause}", True)
	Call skapi.Hotkey("{Break}", True)
	
End Function

Public Function OnHotkey(cmid, vk)
	
'	Handle our hotkeys.
	Select Case cmid
		
	Case "{Pause}"
	'	Pause was pressed.  If a command is running, pause it.
	'	If paused, resume.
		Select Case pstate
		Case pstateRunning
			pstate = pstatePausing
		Case pstatePaused
			pstate = pstateRunning
		End Select
		
	Case "{Break}"
	'	Ctrl+Break was pressed.  Set an error state to stop the running command.
		If fStopOnCtrlBreak Then Call SetError("Command interrupted.")
		
	End Select
	
End Function

Public Function OnCombatMode(fOn)
	
'	Cause an error on entering combat mode if so configured.
	If fOn And fStopOnCombatMode Then Call SetError("Command interrupted by combat mode.")
	
End Function

Public Function OnEnd3D()
	
'	Cause an error on logout if so configured.
	If fStopOnLogout Then Call SetError("Command interrupted by logout.")
	
End Function

Public Function OnDeathSelf(acoKiller, szMsg)
	
'	Cause an error on death if so configured.
	If fStopOnDeath Then Call SetError("Command interrupted by death!")
	
End Function


'----------
' Privates
'----------

' Possible pause states:
Private Property Get pstateRunning()
	pstateRunning = 0		' A command is running.
End Property
Private Property Get pstatePausing()
	pstatePausing = 1		' User has pressed pause.
End Property
Private Property Get pstatePaused()
	pstatePaused = 2		' Script is paused waiting for resume.
End Property
Private Property Get pstateError()
	pstateError = 3		' An error condition has been set.
End Property

Private pstate		' The current pause state.
Private szError		' The message associated with a pending error.


End Class


' Instantiate the global object.
Public Pauser
Set Pauser = New PauserCls


